unit DemoMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ClipBrd, HWSNGen;

type
  TDemoForm = class(TForm)
    IntegrityPnl: TGroupBox;
    IntegrityCheckMemo: TMemo;
    IntegrityCheckBtn: TButton;
    CalculatePnl: TGroupBox;
    ClipBrdBtn: TButton;
    OutputMemo: TMemo;
    CalcBtn: TButton;
    RegCacheCheckbox: TCheckBox;
    MemCacheCheckbox: TCheckBox;
    ClearCacheBtn: TButton;
    RegPassphrase: TEdit;
    RegLabel: TLabel;
    procedure CalcBtnClick(Sender: TObject);
    procedure IntegrityCheckBtnClick(Sender: TObject);
    procedure ClipBrdBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ClearCacheBtnClick(Sender: TObject);
    procedure MemCacheCheckboxClick(Sender: TObject);
    procedure RegCacheCheckboxClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure RegPassphraseChange(Sender: TObject);
  private
    hwGen: THWSNGen;
    procedure ControlState(state: boolean);
  end;

var
  DemoForm: TDemoForm;

implementation

{$R *.dfm}

uses
  MyTimer;

procedure TDemoForm.ControlState(state: boolean);
begin
  CalcBtn.Enabled := state;
  ClearCacheBtn.Enabled := state;
  ClipBrdBtn.Enabled := state;
  RegCacheCheckbox.Enabled := state;
  MemCacheCheckbox.Enabled := state;
  IntegrityCheckMemo.Enabled := state;
  IntegrityCheckBtn.Enabled := state;
  RegPassphrase.Enabled := state;
  RegLabel.Enabled := state;
  OutputMemo.Enabled := state;
  Application.ProcessMessages;
end;

procedure TDemoForm.CalcBtnClick(Sender: TObject);
var
  nZeit: int64;
  timer: TMyTimer;
  hwId: THardwareIdentification;
resourcestring
  LNG_TIME_SPENT = 'Time spent: %f seconds.';
  LNG_RECALC = 'A recalculation is necessary. It will start now.';
  LNG_CACHED = 'The hardware identification can be fetched from a cache (Memory or validated registry key).';
  LNG_SELFTEST_FAILED_MSG = 'The self test has failed! Please send a bug report to ViaThinkSoft!';
  LNG_SELFTEST_OK = 'Hardware Identification Integrity Self-Test PASSED!';
  LNG_SELFTEST_ERR = 'Hardware Identification Integrity Self-Test FAILED!';
  LNG_HWDESC = '= ViaThinkSoft Hardware Identification =' + #13#10#13#10 +
               'Version: %s' + #13#10#13#10 +
               '- Collected Hardware Data -' + #13#10#13#10 +
               '%s' + #13#10#13#10 +
               '- Hashed Hardware Identification -' + #13#10#13#10 +
               '%s' + #13#10#13#10;
begin
  if hwGen.CalculateNecessary then
  begin
    ShowMessage(LNG_RECALC);
  end
  else
  begin
    ShowMessage(LNG_CACHED);
  end;
  ControlState(False);
  timer := TMyTimer.Create;
  try
    timer.StartTimer;
    hwId := hwGen.GetHardwareId();
    OutputMemo.Lines.Text := Format(LNG_HWDESC, [hwGen.GeneratorVersion, hwId.HwCollection, hwId.HwHash]);
    if hwGen.IsValidHWID(hwId.HwHash) then
    begin
      OutputMemo.Lines.Text := OutputMemo.Lines.Text + LNG_SELFTEST_OK + #13#10;
    end
    else
    begin
      OutputMemo.Lines.Text := OutputMemo.Lines.Text + LNG_SELFTEST_ERR + #13#10;
      ShowMessage(LNG_SELFTEST_FAILED_MSG);
    end;

    IntegrityCheckMemo.Lines.Text := hwId.HwHash;
    timer.StopTimer;
    nZeit := timer.Time;
  finally
    timer.Free;
  end;
  ControlState(True);
  ShowMessageFmt(LNG_TIME_SPENT, [nZeit/1000]);
end;

procedure TDemoForm.IntegrityCheckBtnClick(Sender: TObject);
resourcestring
  LNG_HWID_OK = 'This hardware identification is valid.';
  LNG_HWID_ERR = 'This is NOT a valid hardware identification number.';
begin
  if hwGen.IsValidHWID(IntegrityCheckMemo.Text) then
  begin
    ShowMessage(LNG_HWID_OK);
  end
  else
  begin
    ShowMessage(LNG_HWID_ERR);
  end;
end;

procedure TDemoForm.ClipBrdBtnClick(Sender: TObject);
begin
  ClipBoard.AsText := OutputMemo.Text;
end;

procedure TDemoForm.FormCreate(Sender: TObject);
begin
  hwGen := THWSNGen.Create(Self);
end;

procedure TDemoForm.FormDestroy(Sender: TObject);
begin
  hwGen.Free;
end;

procedure TDemoForm.ClearCacheBtnClick(Sender: TObject);
begin
  HwGen.ClearCache;
end;

procedure TDemoForm.MemCacheCheckboxClick(Sender: TObject);
begin
  HwGen.MemoryCache.Enabled := MemCacheCheckbox.Checked;
end;

procedure TDemoForm.RegCacheCheckboxClick(Sender: TObject);
begin
  try
    HwGen.RegistryCache.Enabled := RegCacheCheckbox.Checked;
  except
    RegCacheCheckBox.Checked := HwGen.RegistryCache.Enabled;
    raise;
  end;
end;

procedure TDemoForm.FormShow(Sender: TObject);
begin
  MemCacheCheckbox.Checked := HwGen.MemoryCache.Enabled;
  RegCacheCheckbox.Checked := HwGen.RegistryCache.Enabled;
  RegLabel.Caption := hwGen.RegistryCache.RegistryKey;
end;

procedure TDemoForm.RegPassphraseChange(Sender: TObject);
begin
  hwGen.RegistryCache.IntegrityPassphrase := RegPassphrase.Text;
  RegLabel.Caption := hwGen.RegistryCache.RegistryKey;
end;

end.
